/**
  *  \file client/tiles/planetcolonist.cc
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/tiles/planetcolonist.h"
#include "ui/widgets.h"
#include "util/nls.h"
#include "cpluslib/format.h"
#include "game/planet.h"
#include "util/misc.h"
#include "game/tables.h"
#include "game/team.h"
#include "game/racename.h"

/************************** WPlanetColonistTile **************************/

/** \class WPlanetColonistTile
    \brief Planet Colonists Tile

    Displays information about the planetary colony.

    <pre>
    Population:    XX
    Tax Rate:      XX%
    Colonists are XYZ.
    </pre> */

WPlanetColonistTile::WPlanetColonistTile(GObjectType& type)
    : WObjectTypeObserverTile(GfxPoint(315, 3 * font_heights[FONT_NORMAL]), type)
{
    setLayout(RightJustified, 0, true);
    addControl(GfxRect(0, 3 * font_heights[FONT_NORMAL] - 20, 20, 20), h.add(new UIButton("T", 't')));
}

void
WPlanetColonistTile::drawPlanetColonistTile(UIAbstractTableWriter& w)
{
    w.newLColumn();
    w.writeFixedCell(UIColor::tc_Static, _("Population:"));
    w.writeFixedCell(UIColor::tc_Static, _("Tax Rate:"));
    w.writeFixedCell(UIColor::tc_Static, _("Colonists are"));

    GPlanet* p = dynamic_cast<GPlanet*>(getObject());
    if (p != 0 && p->hasPlanetData()) {
        w.newLColumn(200);
        w.writeVariableCell(UIColor::tc_Green, clansToString(p->getColonists()));
        w.writeVariableCell(UIColor::tc_Green, format("%d%%", p->getColonistTax()));

        int happy = p->getColonistHappiness();
        UISkin::Color c = (happy <= 40
                           ? UIColor::tc_Red
                           : happy < 70
                           ? UIColor::tc_Yellow
                           : UIColor::tc_Green);
        w.writeVariableCell(c, format("%s (%d)", getHappinessName(happy), happy));
    }

    w.finish();
}

void
WPlanetColonistTile::drawData(GfxCanvas& can)
{
    UITableWriter tw(can, getSkin(), getInnerSize(5, 20), FONT_NORMAL);
    drawPlanetColonistTile(tw);
}



WNarrowPlanetColonistTile::WNarrowPlanetColonistTile(GObjectType& type)
    : WObjectTypeObserverTile(GfxPoint(250, 3*font_heights[FONT_NORMAL]), type)
{ }

void
WNarrowPlanetColonistTile::drawData(GfxCanvas& can)
{
    GPlanet* pl = dynamic_cast<GPlanet*>(getObject());
    if (!pl || !pl->isOwnerKnown()) {
        getSkin().drawBackground(can, getExtent());
        return;
    }

    UISkin& sk = getSkin();
    int x = getExtent().x;
    int y = getExtent().y;
    int w = getExtent().w;

    /* Anyone on this planet? */
    if (pl->getOwner() == 0) {
        /* No. */
        sk.setColor(UIColor::tc_Static);
        sk.outTextF(can, FONT_NORMAL, x, y, w, _("No colonists."));
        return;
    }

    /* Normal display:
         Race, 10000 clans
         They like your leadership.
         Taxes: 99% (100 mc) */
    switch (getPlayerRelation(pl->getOwner())) {
     case is_Me:    sk.setColor(UIColor::tc_Green);  break;
     case is_Enemy: sk.setColor(UIColor::tc_Red);    break;
     case is_Ally:  sk.setColor(UIColor::tc_Yellow); break;
     default:       sk.setColor(UIColor::tc_Static); break;
    }
    if (pl->getColonists().isKnown()) {
        sk.outTextF(can, FONT_NORMAL, x, y, w, format(_("%s, %d clan%!1{s%}"),
                                                      player_racenames.getShortName(pl->getOwner()),
                                                      pl->getColonists()));
    } else {
        sk.outTextF(can, FONT_NORMAL, x, y, w, player_racenames.getShortName(pl->getOwner()));
    }
    y += font_heights[FONT_NORMAL];

    if (pl->getColonistHappiness().isKnown()) {
        int happy = pl->getColonistHappiness();
        sk.setColor(happy <= 40
                    ? UIColor::tc_Red
                    : happy < 70
                    ? UIColor::tc_Yellow
                    : UIColor::tc_Green);
        sk.outTextF(can, FONT_NORMAL, x, y, w, format("%s (%d)", getHappinessName(happy), happy));
    }
    y += font_heights[FONT_NORMAL];

    if (pl->getColonistTax().isKnown()) {
        const char* txt = _("Tax Rate:");
        const int wi = fonts[FONT_NORMAL]->getTextWidth(txt) + font_ems[FONT_NORMAL];
        sk.setColor(UIColor::tc_Static);
        sk.outText(can, FONT_NORMAL, x, y, txt);
        sk.setColor(UIColor::tc_Green);
        sk.outTextF(can, FONT_NORMAL, x+wi, y, w-wi, format("%d%%", pl->getColonistTax()));
    }
}
