/**
  *  \file client/tiles/planetnatives.cc
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/tiles/planetnatives.h"
#include "game/planet.h"
#include "ui/uiglobal.h"
#include "game/tables.h"
#include "cpluslib/format.h"
#include "util/nls.h"
#include "ui/table.h"
#include "util/misc.h"
#include "ui/util.h"

/*************************** WPlanetNativeTile ***************************/

/** \class WPlanetNativeTile
    \brief Planet Native Tile

    Displays information about a planet's natives, if any. */

WPlanetNativeTile::WPlanetNativeTile(GObjectType& type)
    : WObjectTypeObserverTile(GfxPoint(315, 4 * font_heights[FONT_NORMAL]), type)
{ }

void
WPlanetNativeTile::drawData(GfxCanvas& can)
{
    GPlanet* p = dynamic_cast<GPlanet*>(getObject());
    if (!p || !p->hasPlanetData())
        return;

    const int y = font_heights[FONT_NORMAL];

    setColor(UIColor::tc_Static);

    if (!p->getNativeRace()) {
        GfxRect r = getInnerSize(5, 5);
        getSkin().setSolid(false);
        getSkin().getParameters().font = fonts[FONT_NORMAL];
        drawBackground(can, GfxRect(5, 0, extent.w-5, 4*y));
        outTextFormattedF(can, getSkin(), r.x, r.y, r.w, 2,
                          _("There are no natives on this planet."));
    } else {
        // This erases the sentence from above because the table
        // writer doesn't (can't?) do that
        drawBackground(can, GfxRect(5, 0, extent.w-5, 2*y));

        UITableWriter tw(can, getSkin(), getInnerSize(5, 0), FONT_NORMAL);

        tw.newLColumn();
        tw.writeFixedCell(UIColor::tc_Static, _("Race:"));
        tw.writeFixedCell(UIColor::tc_Static, _("Population:"));
        tw.writeFixedCell(UIColor::tc_Static, _("Tax Rate:"));
        tw.writeFixedCell(UIColor::tc_Static, _("Natives are"));

        tw.newLColumn(200);
        tw.writeVariableCell(UIColor::tc_Green, string_t(getNativeRaceName(p->getNativeRace())) + ", " + getNativeGovernmentName(p->getNativeGovernment()));
        tw.writeVariableCell(UIColor::tc_Green, clansToString(p->getNatives()));
        tw.writeVariableCell(UIColor::tc_Green, format("%d%%", p->getNativeTax()));

        int happy = p->getNativeHappiness();
        UISkin::Color c = (happy <= 40
                           ? UIColor::tc_Red
                           : happy < 70
                           ? UIColor::tc_Yellow
                           : UIColor::tc_Green);
        tw.writeVariableCell(c, format("%s (%d)", getHappinessName(happy), happy));

        tw.finish();
    }
}



WNarrowPlanetNativeTile::WNarrowPlanetNativeTile(GObjectType& type)
    : WObjectTypeObserverTile(GfxPoint(250, 4 * font_heights[FONT_NORMAL]), type)
{ }

void
WNarrowPlanetNativeTile::drawData(GfxCanvas& can)
{
    GPlanet* pl = dynamic_cast<GPlanet*>(getObject());
    if (!pl || !(pl->getNativeRace().isKnown() || pl->isKnownToHaveNatives())) {
        getSkin().drawBackground(can, getExtent());
        return;
    }

    UISkin& sk = getSkin();
    int x = getExtent().x;
    int y = getExtent().y;
    int w = getExtent().w;

    /* Anyone on this planet? */
    if (!pl->getNativeRace().isKnown()) {
        /* This means this planet is known to have natives, but we don't know which ones. */
        sk.setColor(UIColor::tc_Static);
        sk.outTextF(can, FONT_NORMAL, x, y, w, _("Planet has natives."));
        return;
    }

    if (pl->getNativeRace() == 0) {
        /* No natives. */
        sk.setColor(UIColor::tc_Static);
        sk.outTextF(can, FONT_NORMAL, x, y, w, _("No natives."));
        return;
    }

    /* Normal display:
         Race, 10000 clans
         Government
         They like your leadership.
         Taxes: 99% (100 mc) */
    sk.setColor(UIColor::tc_Green);
    if (pl->getNatives().isKnown()) {
        sk.outTextF(can, FONT_NORMAL, x, y, w, format(_("%s natives, %d clan%!1{s%}"),
                                                      getNativeRaceName(pl->getNativeRace()),
                                                      pl->getNatives()));
    } else {
        sk.outTextF(can, FONT_NORMAL, x, y, w, getNativeRaceName(pl->getNativeRace()));
    }
    y += font_heights[FONT_NORMAL];

    if (pl->getNativeGovernment().isKnown()) {
        sk.setColor(UIColor::tc_Green);
        sk.outTextF(can, FONT_NORMAL, x, y, w, getNativeGovernmentName(pl->getNativeGovernment()));
    } else {
        sk.setColor(UIColor::tc_Static);
        sk.outTextF(can, FONT_NORMAL, x, y, w, _("Unknown government."));
    }
    y += font_heights[FONT_NORMAL];

    if (pl->getNativeHappiness().isKnown()) {
        int happy = pl->getNativeHappiness();
        sk.setColor(happy <= 40
                    ? UIColor::tc_Red
                    : happy < 70
                    ? UIColor::tc_Yellow
                    : UIColor::tc_Green);
        sk.outTextF(can, FONT_NORMAL, x, y, w, format("%s (%d)", getHappinessName(happy), happy));
    }
    y += font_heights[FONT_NORMAL];

    if (pl->getNativeTax().isKnown()) {
        const char* txt = _("Tax Rate:");
        const int wi = fonts[FONT_NORMAL]->getTextWidth(txt) + font_ems[FONT_NORMAL];
        sk.setColor(UIColor::tc_Static);
        sk.outText(can, FONT_NORMAL, x, y, txt);
        sk.setColor(UIColor::tc_Green);
        sk.outTextF(can, FONT_NORMAL, x+wi, y, w-wi, format("%d%%", pl->getNativeTax()));
    }
}
