/**
  *  \file int/interror.cc
  *  \brief Interpreter: Error messages
  *
  *  The basic idea is to generate most error messages by explicit
  *  calls to constructor methods (e.g. IntError::rangeError()). This
  *  allows us to associate a little bit of meta-information with
  *  errors, and generate the final message when the error is
  *  presented to the user.
  *
  *  Note that the backtrace of an error can be obtained from the
  *  process' state and need not be stored with the error message.
  *
  *  TODO: if a user process does
  *      Try / ... / Else / Throw System.Error / EndTry
  *  it would lose this meta-information because the error message is
  *  converted to a string inbetween.
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "int/interror.h"

/******************************** IntError *******************************/

IntError::IntError(const char* error)
    : error(error)
{ }

IntError::IntError(string_t error)
    : error(error)
{ }

IntError::~IntError() throw()
{ }

const char*
IntError::what() const throw()
{
    return error.c_str();
}

void
IntError::addTrace(string_t line)
{
    if (trace.size())
        trace += '\n';
    trace += line;
}

/** Generate an "unknown identifier" error for a given identifier. */
IntError
IntError::unknownIdentifier(string_t name)
{
    // FIXME: make a version that takes the identifier from the executing BCO
    return IntError("Unknown identifier, " + name);
}

/** Generate a "type mismatch" error. */
IntError
IntError::typeError(ExpectedType expectedType)
{
    // FIXME
    static const char*const suffix[] = {
        "",
        ", expecting integer",
        ", expecting bool",
        ", expecting number",
        ", expecting number or string",
        ", expecting string",
        ", expecting list or function",
        ", expecting list",
        ", expecting record",
        ", expecting function or subroutine",
        ", expecting subroutine"
        ", expecting keymap"
    };
    return IntError(string_t("Type error") + suffix[expectedType]);
}

/** Generate an internal error. */
IntError
IntError::internalError(const char* msg)
{
    // FIXME
    return IntError(string_t("Internal error: ") + msg);
}

/** Generate an "object not serializable" error. This error is
    generated when an object incapable of serializing is asked to
    serialize (IntValue::store), which normally only happens when a
    process suspends. */
IntError
IntError::notSerializable()
{
    return IntError("Not suspendable");
}

/** Generate a "not assignable" error. This error is generated when an
    assignment is attempted to a value which is not assignable
    (usually, a read-only property). */
IntError
IntError::notAssignable()
{
    return IntError("Attempt to assign read-only value");
}

/** Generate a range error. */
IntError
IntError::rangeError()
{
    return IntError("Range error");
}

/** Generate a "multi-line not allowed" error. */
IntError
IntError::invalidMultiline()
{
    return IntError("Multi-line statement not allowed here");
}

/** Generate a "expecting keyword" error, one permitted keyword. */
IntError
IntError::expectKeyword(const char* kw)
{
    return IntError(string_t("Expecting '") + kw + "'");
}

/** Generate a "expecting keyword" error, two permitted keywords. */
IntError
IntError::expectKeyword(const char* kw1, const char* kw2)
{
    return IntError(string_t("Expecting '") + kw1 + "' or '" + kw2 + "'");
}

/** Generate a "expecting symbol" error, one permitted keyword. */
IntError
IntError::expectSymbol(const char* sym)
{
    return expectKeyword(sym);
}

/** Generate a "expecting symbol" error, two permitted keywords. */
IntError
IntError::expectSymbol(const char* sym1, const char* sym2)
{
    return expectKeyword(sym1, sym2);
}

/** Generate a "keyword not expected here" error. */
IntError
IntError::misplacedKeyword(const char* kw)
{
    return IntError(string_t("'") + kw + "' not allowed here");
}

/** Generate an "expecting end of line" error. */
IntError
IntError::garbageAtEnd(bool expression)
{
    return IntError(expression
                    ? "Expression incorrectly terminated (missing operator?)"
                    : "Expecting end of line");
}

/** Generate an "expecting identifier" error.
    \param what What the identifier will name */
IntError
IntError::expectIdentifier(const char* what)
{
    return IntError(string_t("Expecting identifier, ") + what);
}

/** Generate a "command not valid in this context" error. */
IntError
IntError::contextError()
{
    return IntError("Command not valid in this context");
}
